;//-----------------------------LICENSE NOTICE------------------------------------
;//  This file is part of Project AcReturns
;//  Copyright (C) 2018 AcReturns
;//
;//  This program is free software: you can redistribute it and/or modify
;//  it under the terms of the GNU General Public License as published by
;//  the Free Software Foundation, either version 3 of the License, or
;//  (at your option) any later version.
;//
;//  This program is distributed in the hope that it will be useful,
;//  but WITHOUT ANY WARRANTY; without even the implied warranty of
;//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;//  GNU General Public License for more details.
;//
;//  You should have received a copy of the GNU General Public License
;//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;//-------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "collisionEnemy.h.s"
.include "entity.h.s"
.include "map.h.s"

;;===============================================================
;;===============================================================
;; PRIVATE DATA
;;===============================================================
;;===============================================================

;.globl _g_level3
.globl restart
.globl _song_inkey
.globl _song_indoor


counter_bc: .dw #0x0000  
map_w:  .dw #0x0028                           ;; 40 in decimal, map width  
tile_w: .db #02                               ;; range of values = 2 -> (X = 0 AND X = 1, == TILE 0), 2px of width per tile
tile_h: .db #04                               ;; range of values = 4 -> (Y = 0 AND Y = 1 AND Y = 2 AND Y = 3, == TILE 0), 4 px of high per tile


.macro AddWidthOrHigh name, width_or_high, left_or_right_and_up_or_down
  bit   #0, left_or_right_and_up_or_down  
  jr    z, up'name                            ;; Rename the tags for no repeats
        ;; ELSE Down plus high
        add   a, width_or_high                ;; A = A + e_h
  up'name:
  bit   #1, left_or_right_and_up_or_down
  jr    z, left'name                          ;; Rename the tags for no repeats
        ;; ELSE right plus width
        add   a, width_or_high                ;; A = A + e_w

  left'name:
  bit   #2, left_or_right_and_up_or_down
  jr    z, end'name

        ;; ELSE Down plus high
        add   a, width_or_high                ;; A = A + e_w
       
  end'name:
  bit   #3, left_or_right_and_up_or_down
  jr    z, end2'name
        ;; ELSE Down plus high
        inc   a
        inc   a

  end2'name:
.endm


.macro check_tile name
  ld    hl, #decompress_buffer                                  ;; HL = Save the pointer of vector of the map
  call  conversor_x_y_in_tiles_and_check_collision      ;; Convert in tiles up-right corner of the hero
  bit   #7, (hl)                                        ;; (HL) number of tile, look the left bit of the byte (HL)
  jp    nz, collision                                   ;; if ((hl) bit 7 != 0 ) then  collision
        bit   #6, (hl)  
        jp   nz, door                                   ;; probar con jr despues
              bit   #5, (hl)
              jp   nz, key
  
  continue'name:
.endm


;;===============================================================
;;===============================================================
;; PUBLIC FUNCTIONS
;;===============================================================
;;===============================================================


;;=========================================================
;; colision_check obstacle of two entity
;; DESTROYS: A
;; INPUT: 
;;  IX = Pointer to first entity hero (e1)
;;  IY = Pointer to second entity obs (e2)
;; OUTPUT:  
;;  A == 0 if collision
;;  A == 1 if not collision
;;=========================================================
collision_check::
  ;; Saber si esta detras, o esta delante no hay colision 
  ;;
  ;; if (e1_x + e1_w <= e2_x) no_collision (izquerda, el obstaculo)
  ;;
  ;; e1_x + e1_w - e2_x <= 0
  ld    iy, #00
  add   iy, de
  ld    a, de_x0(iy)                          ;; A = e1_x
  add   de_w(iy)                              ;; A = A + e1_w (e1_x + e1_w)
  sub   de_x0(ix)                             ;; A = A - e2_x ((e1_x + e1_w) - (e2_x)
  jr    z, no_collision                       ;; if (A == 0) then no_collision
  jp    m, no_collision                       ;; if (A < 0) then no_collision
  
  ;;
  ;; if (e2_x + e2_w <= e1_x) (derecha, el obstaculo)
  ;;
  ;; e2_x + e2_w - e1_x <= 0
  ld    a, de_x0(ix)                          ;; A = e2_x
  add   de_w(ix)                              ;; A = A + (iy + e2_w), (e2_x + e2_w)
  sub   de_x0(iy)                             ;; A = A - (ix + e1_x), ((e2_x + e2_w) - e1_x)
  jr    z, no_collision                       ;; if (A == 0) then no_collision
  jp    m, no_collision                       ;; if (A < 0) then no_collision

  ;; collision in axis X then check collision axis Y
  ;;
  ;;
  ;; if (e1_y + e1_h <= e2_y) (arriba, el obstaculo)
  ;;
  ;; e1_y + e1_h - e2_y <= 0
  ld    a, de_y0(iy)                          ;; A = e1_y
  add   de_h(iy)                              ;; A = A + e1_h, (e1_y + e1_h)
  sub   de_y0(ix)                             ;; A   = A - e2_y, ((e1_y + e1_h) - e2_y)
  jr    z, no_collision                       ;; if (A == 0) then no_collision
  jp    m, no_collision                       ;; if (A < 0) then no_collision

  ;;
  ;; if (e2_y + e2_h <= e1_y) (arriba, el obstaculo)
  ;;
  ;; e2_y + e2_h - e1_y <= 0
  ld    a, de_y0(ix)                          ;; A = e2_y
  add   de_h(ix)                              ;; A = A + e2_h, (e2_y + e2_h)
  sub   de_y0(iy)                             ;; A = A - e1_y, ((e2_y + e2_h) - e1_y)
  jr    z, no_collision                       ;; if (A == 0) then no_collision
  jp    m, no_collision                       ;; if (A < 0) then no_collision

  ;; Collision
  ld    a, #0xFF
  ret

  ;; No Collision
  no_collision:
    ld    a, #0x0F
    ret

;;=========================================================
;; colision_check obstacle of two entity
;; DESTROYS: HL, BC, IY, IX
;; INPUT: 
;;  IX = Pointer to second entity hero (e2)
;; OUTPUT:  
;;=========================================================
collision_tester::
  ;ld    b,  de_x0(ix)               ;; D = de_x(hero)
  ;ld    c,  de_y0(ix)               ;; E = dE_y(hero)
  ;call  ent_move                  ;; Do movement
  call collision_check
  ld    hl,  #0xC000;                
  ld    (hl), a                               ;; Pintamos de color si colisiona (Prueba)
  cp    #0xFF                   
  jr    nz,  no_colision  
        ;;  Collision
        ;ld    de_x0(ix),  b
        ;ld    de_y0(ix),  c
        ;;prueba
        ld    a, #1
        ret
  
  no_colision:
    ;ld    e_vx(ix), #0              ;; E_VX = 0 we are still
   ; ld    e_vy(ix), #0              ;; E_VY = 0 we are still
    ld    a, #0
    ret

;;=========================================================
;; COLLISION WITH THE MAP
;; DESTROYS: HL, BC 
;; INPUT: 
;;  IX = Pointer to hero 
;; OUTPUT: 
;;=========================================================
collision_map::
;; Para avanzar una fila sumar + 40 (map_width)
  ld    hl, #decompress_buffer                        ;; HL = Save the pointer of vector of the map
  ld    b,  de_x0(ix)                         ;; B = de_x(hero)
  ld    c,  de_y0(ix)                         ;; C = dE_y(hero)
  push  bc                                    ;; Save BC in stack
  call  ent_move                              ;; We predict the move of Hero

  ld  a, (axis)
  cp  #0
  jp   nz, check_collision_axisY
       ;ld    bc, counter_bc
      ; d para el contador
      ld    bc, #0x0000

      check_tile name1

       ;ld    bc, #0x0001

     ; check_tile name3

      ld    bc, #0x0003

      check_tile name2

      ;ld    bc, #0x0007

      ;check_tile name3

      ld    bc, #0x000F

      check_tile name3

       

      ld    bc, #0x070F

      check_tile name4

       ;ld    bc, #0x0707

      ;check_tile name3


      ld    bc, #0x0703

      check_tile name5

       ;ld    bc, #0x0701

      ;check_tile name3

      ld    bc, #0x0700

      check_tile name6

     

      ;; convertir a la siguiente posicion
      ;check_tile name3, (counter_bc)
      ;ld    bc, (counter_bc)
    ;
    ;  ;set   #1, c
    ;
      ;ld    (counter_bc), bc

     




      ;volver arriba
      
        ;; ELSE NO Collision
        jp  end


  check_collision_axisY::

    ld    bc, #0x0700

    check_tile name7

    ld    bc, #0x0300

    check_tile name8

    ld    bc, #0x0100

    check_tile name9

    ld    bc, #0x0000

    check_tile name10


      ld    bc, #0x000F

    check_tile name11

     ld    bc, #0x010F

    check_tile name12

    ld    bc, #0x030F

    check_tile name13

    ld    bc, #0x070F

    check_tile name14

  ;; ELSE NO Collision
  jp  end


  ;; Key
  key:

    ;ld    (hl), #16
    ;;;borra la parte de abajo de la llave solo si se coge la parte superior
    ;ld     bc, (map_w)
    ;add   hl, bc
    ;ld    (hl), #16
;
    ;ld    bc, #0xFFd8
    ;add   hl, bc
    ;add   hl, bc
    ;ld    (hl), #16

    ld    a,  e_ch(ix)
    set   #5, a

    ld    e_ch(ix), a                         ;;update charecteristics of entity
    push   ix
    ;ld    ix, #key_data
    ;call  ren_eraseEntity
    ;call  map_switchBuffers
    ;ld    ix, #key_data
    ;call  ren_eraseEntity
    ;call  map_switchBuffers

    ;; Sound of key
    ld    de, #_song_inkey 
    call cpct_akp_musicInit_asm
    call cpct_akp_musicPlay_asm


    ld    ix, #key_data
    ld    hl, #decompress_buffer
    ld    bc, #00
    call  conversor_x_y_in_tiles_and_check_collision
    ld    (hl), #16
    ld    bc, #0x0028
    add   hl, bc
    ld    (hl), #16
    call  ren_eraseEntity
    call  map_switchBuffers

    call  ren_eraseEntity
    call  map_switchBuffers
    pop ix

    pop   bc                          ;; We restore BC, B = de_x(hero), C = dE_y(hero)
    ld    de_x0(ix),  b                 ;; Update the de_x of the hero
    ld    de_y0(ix),  c                 ;; Update the dE_y of the hero
    ld    e_vx(ix), #0                      ;; E_VX = 0 we are still
    ld    e_vy(ix), #0 
 ;   push  bc

    ld    a, #0
    ret

  ;;Door
  door:
    ld    a, e_ch(ix)                         ;; Comprobamos si es tiene la llave el hero
    bit   #5, a
    jr    z, not_have_key   ;,
          ;;open the door

          ;; Sound of door with key
          ld    de, #_song_indoor 
          call cpct_akp_musicInit_asm
          call cpct_akp_musicPlay_asm
          
          call more_points
          pop bc
          ;ld  hl, #0x8000
          ;ld  de, #0x8001
          ;ld  (hl), #00
          ;ld  bc, #0x8000
          ;ldir
          ld  a,  #99

          call #restart
          ;ret

    not_have_key:
    pop   bc                                  ;; We restore BC, B = de_x(hero), C = dE_y(hero)
    ld    de_x0(ix),  b                       ;; Update the de_x of the hero
    ld    de_y0(ix),  c                       ;; Update the dE_y of the hero
    push  bc                                  ;; Save the BC in the stack
    jr    end2

  ;;Collision
  collision :

    ld  a, (axis)
    cp  #0
    jp   nz, check_collision_axisY_2

      pop   bc                                  ;; We restore BC, B = de_x(hero), C = dE_y(hero)
      ld    de_x0(ix),  b                       ;; Update the de_x of the hero
      ;ld    de_y0(ix),  c                       ;; Update the dE_y of the hero
      push  bc                                   ;; Save the BC in the stack
      jr    end2

    check_collision_axisY_2:
      pop   bc                                  ;; We restore BC, B = de_x(hero), C = dE_y(hero)
      ;ld    de_x0(ix),  b                       ;; Update the de_x of the hero
      ld    de_y0(ix),  c                       ;; Update the dE_y of the hero
      push  bc 

      jr    end2
      
      
    end:
      ;pop   bc                              ;; We restore BC, B = de_x(hero), C = dE_y(hero)  
    ;push bc
     ;call collision_tester
     ld a, #0  
     or   a
     jr   z, end2        ;; A == 0 then no collision                  
          ;;collision

          pop  bc
          ld  a, (axis)
          cp  #0
          jp   nz, check_collision_axisY_3

                                             ;; We restore BC, B = de_x(hero), C = dE_y(hero)
            ld    de_x0(ix),  b                       ;; Update the de_x of the hero
            ld    e_vx(ix), #0                  ;; E_VX = 0 we are still
            ;ld    de_y0(ix),  c                       ;; Update the dE_y of the hero
                                              ;; Save the BC in the stack
            ret

          check_collision_axisY_3:
                                             ;; We restore BC, B = de_x(hero), C = dE_y(hero)
            ;ld    de_x0(ix),  b                       ;; Update the de_x of the hero
            ld    de_y0(ix),  c                       ;; Update the dE_y of the hero
             ld    e_vy(ix), #0 
          
          ret

     end2:
      pop bc
      ld  a, (axis)
      cp  #0
      jp   nz, check_collision_axisY_4

          ld    e_vx(ix), #0                      ;; E_VX = 0 we are still
          ld a,#0
          ret

      check_collision_axisY_4:
      ld    e_vy(ix), #0                      ;; E_VY = 0 we are still
      ld a,#0
      ret


;;===============================================================
;;===============================================================
;; PRIVATE FUNCTIONS
;;===============================================================
;;===============================================================


;;=========================================================
;; CONVERT IN TIILES, (X,Y) OF THE HERO
;;
;; DESTROYS: A, HL, BC, D
;;
;; INPUT: 
;;  IX = Pointer to hero 
;;  HL = Pointer of vector of the map
;;
;;  Axis X
;;  B  = up or down, and left or right 
;;    B == 0000 0000, then up, (BIT O)
;;    B == 0000 0001, then down,  (BIT O)
;;    B == 0000 0000, then left,  (BIT 1)
;;    B == 0000 0010, then right,  (BIT 1)
;;
;;  Axis Y
;;  C  = up or down, and left or right 
;;    C == 0000 0000, then up, (BIT O)
;;    C == 0000 0001, then down,  (BIT O)
;;    C == 0000 0000, then left,  (BIT 1)
;;    C == 0000 0010, then right,  (BIT 1)
;;  
;;
;; OUTPUT: 
;;    Flag Z = 1 if (no collision)
;;         Z = 0 if (collision)
;;=========================================================
conversor_x_y_in_tiles_and_check_collision::
  ld    a, (tile_w)                           ;; A = tile_w in px
  ld    d, a                                  ;; D = A, (tile_w in px)
  ld    a, de_x0(ix)                          ;; A = de_x(hero)
  ;;de_w(ix), #02
  AddWidthOrHigh name1, #01, b
  
  continue_sub:
    sub   d                                   ;; A = A - D, de_x(hero) - tile_w
    jp    m, calcule_y                        ;; if (de_x(hero) - tile_w < 0) then jp calculde_y
          ;; ELSE It is not negative
          inc   hl                            ;; HL++,  progress through the vector
          jr    continue_sub            

  calcule_y:
  ld    a, (tile_h)                           ;; A = tile_h in px
  ld    d, a                                  ;; D = A, (tile_h in px)
  ld    a, de_y0(ix)                          ;; A = dE_y(hero)
  ;;  de_h(ix), #04
  
  AddWidthOrHigh name2, #04, c

  continue_calcule_y:
    sub   d                                   ;; A = A - D, dE_y(hero) - tile_h
    jp    m, fin                              ;; if (dE_y(hero) - tile_h < 0) then jp fin
          ;; ELSE It is not negative
          ld    bc, (map_w)                   ;; BC = 40 in decimal, map width  
          add   hl, bc                        ;; HL = HL + BC, progress through the vector, next row
          jr    continue_calcule_y            ;; jp continue_calcule_y

  fin:
  ret