;//-----------------------------LICENSE NOTICE------------------------------------
;//  This file is part of Project AcReturns
;//  Copyright (C) 2018 AcReturns
;//
;//  This program is free software: you can redistribute it and/or modify
;//  it under the terms of the GNU General Public License as published by
;//  the Free Software Foundation, either version 3 of the License, or
;//  (at your option) any later version.
;//
;//  This program is distributed in the hope that it will be useful,
;//  but WITHOUT ANY WARRANTY; without even the implied warranty of
;//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;//  GNU General Public License for more details.
;//
;//  You should have received a copy of the GNU General Public License
;//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;//-------------------------------------------------------------------------------


;###########################################################################
;#### FICHERO: main.s
;###########################################################################
;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;###########################################################################
;#### FICHERO: main.s
;###########################################################################
;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
 
;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "entity.h.s"
.include "collision.h.s"
.include "keyboard/keyboard.h.s"
.include "menu.h.s"
.include "map.h.s"
.include "counters.h.s"

.globl restart_dead
.globl run

.include "main.h.s"

.globl restart
.globl ren_eraseEntity
.globl  ren_drawEntity
.globl  ren_updateEntity

.globl _song_ingame
.globl _song_inmenu

.globl _hero_pal             ;; Palette colour of sprite hero
;.globl _tileset                    ;; Tile vector
;.globl _g_level1                        ;; Map vector

.globl _tile64set
;.globl _g_level3
;;
;; Start of _DATA areaok
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA

.area _CODE

isr_count: .db  #0X12

;; Interrup service rutine
isr::
  ex  af, af'
  exx                                   ;; Intercambia todos los registros
  push  af
  push  bc
  push  de
  push  hl
  push  iy

  ld  a,  (isr_count)
  dec a
  ld  (isr_count),  a
  jr  nz, return
        call cpct_akp_musicPlay_asm
        ld  a,  #12                     ;; Music 25Hz
        ld  (isr_count),  a

  return:
  pop  iy
  pop  hl
  pop  de
  pop  bc
  pop  af
  exx
  ex  af, af'

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INICIALIZATION THE GAME
;; DESTROYED REGISTERS:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
inicialization:
  ;; Disable firmware to prevent it from interfering with string drawing
  call  cpct_disableFirmware_asm
 
  ld    c, #0                          ;; Mode video = 0
  call  cpct_setVideoMode_asm
  
  ld    hl,  #_hero_pal          ;; HL pointer to Sprite_palette of hero (src/sprite/hero.c)
  ld    de, #16                        ;; Number of colors (Max 16)
  call  cpct_setPalette_asm

  ; Inizialice music of menu
  ld    de, #_song_inmenu 
  call cpct_akp_musicInit_asm

  ;; Set ISR
  ld    hl, #isr 
  call cpct_setInterruptHandler_asm

  ld	l,	#16
  ld	h,	#0x14									;; Colour Hardware
  call	cpct_setPALColour_asm
  
  call  map_inicialize

  ret
  ;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; FINISH THE LEVEL
;; DESTROYED REGISTERS:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
restart: 
  ;call more_points                             ;; Is load when the game has ended
  call load_endScreen
;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;
_main::
    ld    sp, #0x8000

;;Restaura nivel
run:
  call  inicialization
  call  load_menu

  ld a,#0
  call load_level
  ;; Inizialice music of game
  ld    de, #_song_ingame 
  call cpct_akp_musicInit_asm
  restart_dead:
  call  restart_timer
  call  restauro
  call  load_points
  ;call  redraw_map
  
loop::

  ;call load_pause
  ;; Inicio
  call load_pause
  ;call  ent_update
  ld    hl, #ent_update
  call  ent_doForAll
  cp  a , #99  ;; Collision map returns 99 if collision with door and hero have a key
  jp  z , #restart ;; Resetea

  call  ent_doForAllY

  ;; Fin
  call  cpct_waitVSYNC_asm
  ;; Restart music 
  ld    de, #_song_ingame 
  call cpct_akp_musicInit_asm

  call map_switchBuffers

  call  timer                           ;; start timer
  ;; Loop forever
  jr    loop


  ;; forever loop
  forever_loop:
    jr    forever_loop


check_draw::

  ;call  ent_update
 
  ;ld    a, e_ch(ix)
  ld    a, e_ch(ix)

  bit   #3, a
  jr    z, no_more_draws2   ;; no more draws

    ;;we have 1 draw
    bit   #4, a
    jr    z, only_one_draw2

      ;;we have 2 draw
      res   #4, a
      ;ld    e_ch(ix), a

      jr    do_continue2

    only_one_draw2:

      res   #3, a
      ;ld    e_ch(ix), a

    do_continue2:
      call ren_eraseEntity
      call ren_drawEntity
      call ren_updateEntity

     no_more_draws2:

     ret
      ;ld    ix, #hero_data
      ;call  ent_doForAllY