;//-----------------------------LICENSE NOTICE------------------------------------
;//  This file is part of Project AcReturns
;//  Copyright (C) 2018 AcReturns
;//
;//  This program is free software: you can redistribute it and/or modify
;//  it under the terms of the GNU General Public License as published by
;//  the Free Software Foundation, either version 3 of the License, or
;//  (at your option) any later version.
;//
;//  This program is distributed in the hope that it will be useful,
;//  but WITHOUT ANY WARRANTY; without even the implied warranty of
;//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;//  GNU General Public License for more details.
;//
;//  You should have received a copy of the GNU General Public License
;//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;//-------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "render.h.s"

.globl e_ch 

m_front_buffer: .db #0xC0
m_back_buffer: .db #0x80

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DRAW AN ENTITY
;; DESTROYED REGISTERS: AF, BC, DE, HL
;; INPUT: IX -> Pointer to Entity
;; INPUT: BC -> First position of video memory
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_drawEntity:
  ld    a,  (m_back_buffer)
  ld    d,  a
  ld    e,  #00         					
	ld		c, de_x0(ix)                 	        ;; C = Entity X
	ld		b, de_y0(ix)                 	        ;; B = Entity Y 
  call  cpct_getScreenPtr_asm				          ;; HL = Ptr of memory, of X and Y
 	ex		de, hl                     	          ;; Exchange registers
  ;; Paint Sprite 
  ld		l,	de_sp_l(ix)
  ld		h,	de_sp_h(ix)
  ld		b,	de_h(ix)
  ld		c,	de_w(ix)
  call cpct_drawSprite_asm
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ERASE AN ENTITY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_eraseEntity:
  ld    a,  (m_back_buffer)
  ld    d,  a
  ld    e,  #00                   
  ld    c, de_x2(ix)                          ;; C = Entity X
  ld    b, de_y2(ix)                          ;; B = Entity Y 
  call  cpct_getScreenPtr_asm                 ;; HL = Ptr of memory, of X and Y
  ex    de, hl                                ;; Exchange registers
  ld    a  ,  #0xcc                           ;; Color de fondo
  ld    b , de_h(ix)
  ld    c , de_w(ix)
  call cpct_drawSolidBox_asm                  ;; Draw a Box, pointer to the upper left box corner,
  ret   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; UPDATE AN ENTITY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_updateEntity:
  ;update X
  ld    a, de_x1(ix)
  ld    de_x2(ix), a
  ld    a, de_x0(ix)
  ld    de_x1(ix), a

  ;;update Y
  ld    a, de_y1(ix)
  ld    de_y2(ix), a
  ld    a, de_y0(ix)
  ld    de_y1(ix), a

  ;; decrease number of draws
  ld    a, e_ch(ix)
  bit   #3, a
  jr    z, no_more_draws                      ;; no more draws
        ;;we have 1 draw
        bit   #4, a
        jr    z, only_one_draw
              ;;we have 2 draw
              res   #4, a
              ld    e_ch(ix), a
              ret

        only_one_draw:
        res   #3, a
        ld    e_ch(ix), a

  no_more_draws:
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CAMBIA EL PUNTERO DE COMIENZO DE MEMORIA DE VIDEO
;; DESTROYED REGISTERS: F, BC, HL , A
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
map_switchBuffers::
  ld  a,  (m_back_buffer)
  ld  b,  a
  ld  a,  (m_front_buffer)
  ld  (m_back_buffer),  a
  ld  a,  b
  ld  (m_front_buffer), a

  srl b                                       ;; Desplazamos el primer bit 1 a la derecha
  srl b                                       ;; Desplazamos el segundo bit 1 a la derecha
  ld  l,  b
  call cpct_setVideoMemoryPage_asm
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PUNTERO AL BACKBUFFER
;; DESTROYED REGISTERS: A
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
getBackBuffer::
    ld  a,  (m_back_buffer)
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PUNTERO AL FRONTBUFFER
;; DESTROYED REGISTERS: A
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
getFrontBuffer::
    ld  a,  (m_front_buffer)
    ret